use std::net::SocketAddr;

use actix_web::{App, web, HttpServer};
use clap::Parser;

use nom_nom_gc::handlers;
use nom_nom_gc::models::read_config;
use nom_nom_gc::models;

#[derive(Parser, Debug)]
#[command(author, version, about, long_about = None)]
struct CLIArgs {
    #[arg(short, long)]
    bind: String,
    #[arg(short, long)]
    config: Option<String>
}

#[tokio::main]
async fn main() -> std::io::Result<()> {
    let args = CLIArgs::parse();
    let addr: SocketAddr = args.bind.parse().unwrap_or_else(|_| panic!("Cannot bind to {}. Please provide a host and port like [::1]:8000", &args.bind));
    let config_path = args.config.unwrap_or("/etc/nom-nom-gc/config.json".to_owned());
    let config = read_config(&config_path)
        .unwrap_or_else(|e| panic!("Cannot read config file: {}", e.to_string()));
    let migration_state = models::AppState::new(config);
    println!("Running DB migrations");
    migration_state.run_migrations().await.unwrap_or_else(|e| panic!("Db migration error: {}", e));

    println!("Server listening to {}", &args.bind);
    HttpServer::new(
        move || {
            let config = read_config(&config_path)
                .unwrap_or_else(|e| panic!("Cannot read config file: {}", e.to_string()));
            let state = models::AppState::new(config);
            App::new().app_data(web::Data::new(state))
                .route("/", web::get().to(handlers::landing_page))
                .route("/account/register/{uuid}", web::get().to(handlers::webauthn_registration))
                .route("/account/register-init", web::post().to(handlers::start_webauthn_registration))
                .route("/account/register-finish", web::post().to(handlers::finish_webauthn_registration))
        })
        .bind(addr)
        .unwrap()
        .run()
        .await
}
