use handlebars::RenderError;
use serde_json::json;
use handlebars::Handlebars;
use uuid::Uuid;

use std::{path::PathBuf, sync::Arc};

pub fn new<'a>() -> Result<Handlebars<'a>, RenderError> {
    let rootpath = PathBuf::from(env!("CARGO_MANIFEST_DIR"));
    let landing_path = rootpath.join("src/templates/landing.hbs");
    let template_path = rootpath.join("src/templates/template.hbs");
    let register_user = rootpath.join("src/templates/register-user.hbs");
    let mut hbs = handlebars::Handlebars::new();
    let css = rootpath.join("src/templates/main.css");
    let webauthn_js = rootpath.join("src/templates/webauthn.js");

    hbs.register_template_file("landing", landing_path.to_str().unwrap())?;
    hbs.register_template_file("template", template_path.to_str().unwrap())?;
    hbs.register_template_file("css", css.to_str().unwrap())?;
    hbs.register_template_file("webauthn-js", webauthn_js.to_str().unwrap())?;
    hbs.register_template_file("register-user", register_user.to_str().unwrap())?;
    Ok(hbs)
}

pub fn landing_page(hb: Arc<Handlebars<'_>>) -> Result<String, RenderError> {
    let data = json!({

    });
    hb.render("landing", &data)
}

pub fn register_user_start(hb: Arc<Handlebars<'_>>, uuid: Uuid, username: String) -> Result<String, RenderError> {
    let uuid = uuid.to_string();
    let js_data = json!({
        "uuid": &uuid,
        "username": username,
    });
    let js = hb.render("webauthn-js", &js_data)?;
    let data = json!({
        "js": js,
        "uuid": &uuid,

    });
    hb.render("register-user", &data)
}
